@@ -24,6 +24,7 @@
 #include "video/VideoSource.h"
 #include "video/VideoRenderer.h"
 #include <atomic>
+#include "video/ScreamCongestionController.h"
 #include "audio/AudioInput.h"
 #include "BlockingQueue.h"
 #include "audio/AudioOutput.h"
@@ -39,7 +40,7 @@
 #include "MessageThread.h"
 #include "utils.h"
 
-#define LIBTGVOIP_VERSION "2.4.2"
+#define LIBTGVOIP_VERSION "2.4.4"
 
 #ifdef _WIN32
 #undef GetCurrentTime
@@ -125,7 +126,7 @@ namespace tgvoip{
 			TCP_RELAY
 		};
 
-		Endpoint(int64_t id, uint16_t port, IPv4Address& address, IPv6Address& v6address, Type type, unsigned char* peerTag);
+		Endpoint(int64_t id, uint16_t port, const IPv4Address& address, const IPv6Address& v6address, Type type, unsigned char* peerTag);
 		Endpoint();
 		~Endpoint();
 		const NetworkAddress& GetAddress() const;
@@ -212,6 +213,9 @@ namespace tgvoip{
 
 			bool logPacketStats=false;
 			bool enableVolumeControl=false;
+
+			bool enableVideoSend=false;
+			bool enableVideoReceive=false;
 		};
 
 		struct TrafficStats{
@@ -415,6 +419,7 @@ namespace tgvoip{
 		float GetOutputLevel(){
 			return 0.0f;
 		};
+		int GetVideoResolutionForCurrentBitrate();
 		void SetVideoSource(video::VideoSource* source);
 		void SetVideoRenderer(video::VideoRenderer* renderer);
 		
@@ -434,6 +439,8 @@ namespace tgvoip{
 			uint16_t id; // for group calls only
 			double sendTime;
 			double ackTime;
+			uint8_t type;
+			uint32_t size;
 		};
 		struct PendingOutgoingPacket{
 			PendingOutgoingPacket(uint32_t seq, unsigned char type, size_t len, Buffer&& data, int64_t endpoint){
@@ -504,8 +511,8 @@ namespace tgvoip{
 		void SendStreamCSD(Stream& stream);
 		void InitializeTimers();
 		void ResetEndpointPingStats();
-		void SendVideoFrame(const Buffer& frame, int32_t flags);
-		void ProcessIncomingVideoFrame(Buffer frame, uint32_t pts);
+		void SendVideoFrame(const Buffer& frame, uint32_t flags);
+		void ProcessIncomingVideoFrame(Buffer frame, uint32_t pts, bool keyframe);
 		std::shared_ptr<Stream> GetStreamByType(int type, bool outgoing);
 		Endpoint* GetEndpointForPacket(const PendingOutgoingPacket& pkt);
 		bool SendOrEnqueuePacket(PendingOutgoingPacket pkt, bool enqueue=true);
@@ -527,6 +534,7 @@ namespace tgvoip{
 			std::shared_ptr<CallbackWrapper> callbackWrapper;
 			std::vector<Buffer> codecSpecificData;
 			bool csdIsValid=false;
+			int resolution;
 			unsigned int width=0;
 			unsigned int height=0;
 			uint16_t rotation=0;
@@ -561,6 +569,16 @@ namespace tgvoip{
 			double timestamp;
 			int32_t length;
 		};
+		struct SentVideoFrame{
+			uint32_t num;
+			uint32_t fragmentCount;
+			std::vector<uint32_t> unacknowledgedPackets;
+            uint32_t fragmentsInQueue;
+		};
+		struct PendingVideoFrameFragment{
+			uint32_t pts;
+			Buffer data;
+		};
 
 		void RunRecvThread();
 		void RunSendThread();
@@ -595,6 +613,9 @@ namespace tgvoip{
 		void TickJitterBufferAngCongestionControl();
 		void ResetUdpAvailability();
 		std::string GetPacketTypeString(unsigned char type);
+		void SetupOutgoingVideoStream();
+		bool WasOutgoingPacketAcknowledged(uint32_t seq);
+		RecentOutgoingPacket* GetRecentOutgoingPacket(uint32_t seq);
 
 		int state;
 		std::map<int64_t, Endpoint> endpoints;
@@ -602,7 +623,7 @@ namespace tgvoip{
 		int64_t preferredRelay=0;
 		int64_t peerPreferredRelay=0;
 		bool runReceiver;
-		uint32_t seq;
+		std::atomic<uint32_t> seq;
 		uint32_t lastRemoteSeq;
 		uint32_t lastRemoteAckSeq;
 		uint32_t lastSentSeq;
@@ -722,6 +743,9 @@ namespace tgvoip{
 		effects::Volume outputVolume;
 		effects::Volume inputVolume;
 
+		std::vector<uint32_t> peerVideoDecoders;
+        int peerMaxVideoResolution=0;
+
 #if defined(TGVOIP_USE_CALLBACK_AUDIO_IO)
 		std::function<void(int16_t*, size_t)> audioInputDataCallback;
 		std::function<void(int16_t*, size_t)> audioOutputDataCallback;
@@ -733,7 +757,24 @@ namespace tgvoip{
 		video::VideoSource* videoSource=NULL;
 		video::VideoRenderer* videoRenderer=NULL;
 		double firstVideoFrameTime=0.0;
-		
+		uint32_t videoFrameCount=0;
+		uint32_t lastReceivedVideoFrameNumber=UINT32_MAX;
+		std::vector<SentVideoFrame> sentVideoFrames;
+		Mutex sentVideoFramesMutex;
+		bool videoKeyframeRequested=false;
+		video::ScreamCongestionController videoCongestionControl;
+		std::vector<PendingVideoFrameFragment> videoPacingQueue;
+		uint32_t sendVideoPacketID=MessageThread::INVALID_ID;
+		uint32_t videoPacketLossCount=0;
+		uint32_t currentVideoBitrate=0;
+		double lastVideoResolutionChangeTime=0.0;
+
+		/*** debug report problems ***/
+		bool wasReconnecting=false;
+		bool wasExtraEC=false;
+		bool wasEncoderLaggy=false;
+		bool wasNetworkHandover=false;
+
 		/*** persistable state values ***/
 		bool proxySupportsUDP=true;
 		bool proxySupportsTCP=true;
