@@ -25,7 +25,7 @@ public:
 	static void EnumerateDevices(std::vector<AudioInputDevice>& devs);
 
 private:
-	void RunThread(void* arg);
+	void RunThread();
 
 	int (*_snd_pcm_open)(snd_pcm_t** pcm, const char* name, snd_pcm_stream_t stream, int mode);
 	int (*_snd_pcm_set_params)(snd_pcm_t* pcm, snd_pcm_format_t format, snd_pcm_access_t access, unsigned int channels, unsigned int rate, int soft_resample, unsigned int latency);
