@@ -6,6 +6,8 @@
 
 #ifdef __ANDROID__
 #include "../os/android/VideoSourceAndroid.h"
+#elif defined(__APPLE__) && !defined(TARGET_OSX32)
+#include "../os/darwin/VideoToolboxEncoderSource.h"
 #endif
 
 using namespace tgvoip;
@@ -20,7 +22,7 @@ std::shared_ptr<VideoSource> VideoSource::Create(){
 }
 
 
-void VideoSource::SetCallback(std::function<void(const Buffer &, int32_t)> callback){
+void VideoSource::SetCallback(std::function<void(const Buffer &, uint32_t)> callback){
 	this->callback=callback;
 }
 
@@ -31,3 +33,12 @@ bool VideoSource::Failed(){
 std::string VideoSource::GetErrorDescription(){
 	return error;
 }
+
+std::vector<uint32_t> VideoSource::GetAvailableEncoders(){
+#ifdef __ANDROID__
+	return VideoSourceAndroid::availableEncoders;
+#elif defined(__APPLE__) && !defined(TARGET_OSX32)
+	return VideoToolboxEncoderSource::GetAvailableEncoders();
+#endif
+	return std::vector<uint32_t>();
+}
